@@ -2151,4 +2151,5 @@ end
 # ARGV is arguable by OptionParser
 ARGV.extend(OptionParser::Arguable)
 
-OptParse = OptionParser
+# An alias for OptionParser.
+OptParse = OptionParser  # :nodoc:
