@@ -4,7 +4,7 @@
 #
 # Author:: Akira Yamada <akira@ruby-lang.org>
 # License:: You can redistribute it and/or modify it under the same term as Ruby.
-# Revision:: $Id: generic.rb 53536 2016-01-14 15:06:37Z naruse $
+# Revision:: $Id: generic.rb 56878 2016-11-22 23:44:51Z kazu $
 #
 # See URI for general documentation
 #
@@ -207,7 +207,7 @@ module URI
           "the scheme #{@scheme} does not accept registry part: #{registry} (or bad hostname?)"
       end
 
-      @scheme.freeze if @scheme
+      @scheme&.freeze
       self.set_path('') if !@path && !@opaque # (see RFC2396 Section 5.2)
       self.set_port(self.default_port) if self.default_port && !@port
     end
@@ -330,7 +330,7 @@ module URI
     # see also URI::Generic.scheme=
     #
     def set_scheme(v)
-      @scheme = v ? v.downcase : v
+      @scheme = v&.downcase
     end
     protected :set_scheme
 
@@ -428,7 +428,7 @@ module URI
 
       if parser.regexp[:USERINFO] !~ v
         raise InvalidComponentError,
-          "bad component(expected user component): #{v}"
+          "bad password component"
       end
 
       return true
@@ -683,7 +683,7 @@ module URI
       if @opaque
         raise InvalidURIError,
           "can not set port with registry or opaque"
-      elsif !v.kind_of?(Fixnum) && parser.regexp[:PORT] !~ v
+      elsif !v.kind_of?(Integer) && parser.regexp[:PORT] !~ v
         raise InvalidComponentError,
           "bad component(expected port component): #{v.inspect}"
       end
@@ -697,7 +697,7 @@ module URI
     # see also URI::Generic.port=
     #
     def set_port(v)
-      v = v.empty? ? nil : v.to_i unless !v || v.kind_of?(Fixnum)
+      v = v.empty? ? nil : v.to_i unless !v || v.kind_of?(Integer)
       @port = v
     end
     protected :set_port
@@ -1096,16 +1096,20 @@ module URI
     #   # =>  #<URI::HTTP:0x2021f3b0 URL:http://my.example.com/main.rbx?page=1>
     #
     def merge(oth)
-      begin
-        base, rel = merge0(oth)
-      rescue
-        raise $!.class, $!.message
+      rel = parser.send(:convert_to_uri, oth)
+
+      if rel.absolute?
+        #raise BadURIError, "both URI are absolute" if absolute?
+        # hmm... should return oth for usability?
+        return rel
       end
 
-      if base == rel
-        return base
+      unless self.absolute?
+        raise BadURIError, "both URI are relative"
       end
 
+      base = self.dup
+
       authority = rel.userinfo || rel.host || rel.port
 
       # RFC2396, Section 5.2, 2)
@@ -1136,31 +1140,6 @@ module URI
     end # merge
     alias + merge
 
-    # return base and rel.
-    # you can modify `base', but can not `rel'.
-    def merge0(oth)
-      oth = parser.send(:convert_to_uri, oth)
-
-      if self.relative? && oth.relative?
-        raise BadURIError,
-          "both URI are relative"
-      end
-
-      if self.absolute? && oth.absolute?
-        #raise BadURIError,
-        #  "both URI are absolute"
-        # hmm... should return oth for usability?
-        return oth, oth
-      end
-
-      if self.absolute?
-        return self.dup, oth
-      else
-        return oth, oth
-      end
-    end
-    private :merge0
-
     # :stopdoc:
     def route_from_path(src, dst)
       case dst
@@ -1486,6 +1465,8 @@ module URI
     # ftp_proxy, no_proxy, etc.
     # If there is no proper proxy, nil is returned.
     #
+    # If the optional parameter, +env+, is specified, it is used instead of ENV.
+    #
     # Note that capitalized variables (HTTP_PROXY, FTP_PROXY, NO_PROXY, etc.)
     # are examined too.
     #
@@ -1494,41 +1475,41 @@ module URI
     # So HTTP_PROXY is not used.
     # http_proxy is not used too if the variable is case insensitive.
     # CGI_HTTP_PROXY can be used instead.
-    def find_proxy
+    def find_proxy(env=ENV)
       raise BadURIError, "relative URI: #{self}" if self.relative?
       name = self.scheme.downcase + '_proxy'
       proxy_uri = nil
-      if name == 'http_proxy' && ENV.include?('REQUEST_METHOD') # CGI?
+      if name == 'http_proxy' && env.include?('REQUEST_METHOD') # CGI?
         # HTTP_PROXY conflicts with *_proxy for proxy settings and
         # HTTP_* for header information in CGI.
         # So it should be careful to use it.
-        pairs = ENV.reject {|k, v| /\Ahttp_proxy\z/i !~ k }
+        pairs = env.reject {|k, v| /\Ahttp_proxy\z/i !~ k }
         case pairs.length
         when 0 # no proxy setting anyway.
           proxy_uri = nil
         when 1
           k, _ = pairs.shift
-          if k == 'http_proxy' && ENV[k.upcase] == nil
+          if k == 'http_proxy' && env[k.upcase] == nil
             # http_proxy is safe to use because ENV is case sensitive.
-            proxy_uri = ENV[name]
+            proxy_uri = env[name]
           else
             proxy_uri = nil
           end
         else # http_proxy is safe to use because ENV is case sensitive.
-          proxy_uri = ENV.to_hash[name]
+          proxy_uri = env.to_hash[name]
         end
         if !proxy_uri
           # Use CGI_HTTP_PROXY.  cf. libwww-perl.
-          proxy_uri = ENV["CGI_#{name.upcase}"]
+          proxy_uri = env["CGI_#{name.upcase}"]
         end
       elsif name == 'http_proxy'
-        unless proxy_uri = ENV[name]
-          if proxy_uri = ENV[name.upcase]
+        unless proxy_uri = env[name]
+          if proxy_uri = env[name.upcase]
             warn 'The environment variable HTTP_PROXY is discouraged.  Use http_proxy.'
           end
         end
       else
-        proxy_uri = ENV[name] || ENV[name.upcase]
+        proxy_uri = env[name] || env[name.upcase]
       end
 
       if proxy_uri.nil? || proxy_uri.empty?
@@ -1545,11 +1526,20 @@ module URI
       end
 
       name = 'no_proxy'
-      if no_proxy = ENV[name] || ENV[name.upcase]
-        no_proxy.scan(/([^:,]*)(?::(\d+))?/) {|host, port|
-          if /(\A|\.)#{Regexp.quote host}\z/i =~ self.host &&
-            (!port || self.port == port.to_i)
-            return nil
+      if no_proxy = env[name] || env[name.upcase]
+        no_proxy.scan(/(?!\.)([^:,\s]+)(?::(\d+))?/) {|host, port|
+          if (!port || self.port == port.to_i)
+            if /(\A|\.)#{Regexp.quote host}\z/i =~ self.host
+              return nil
+            else
+              require 'ipaddr'
+              return nil if
+                begin
+                  IPAddr.new(host)
+                rescue IPAddr::InvalidAddressError
+                  next
+                end.include?(self.host)
+            end
           end
         }
       end
